
#include <LGSM.h>

#include <LBT.h>
#include <LBTServer.h>

#include <LTask.h>
#include "vmthread.h"
#include "stddef.h"
#include <LWiFi.h>
#include <LWiFiClient.h>

#define WIFI_AP "SSID" //put the name of your wireless network here
#define WIFI_PASSWORD ""
#define WIFI_AUTH LWIFI_OPEN  // choose from LWIFI_OPEN, LWIFI_WPA, or LWIFI_WEP.
#define SITE_URL "api.mediatek.com" //the site of the API

#define DEVICEID "XXXXXXXX" // device id 
#define DEVICEKEY "YYYYYYYYYYYYYYYY" // device key 

LWiFiClient connection;


#include <dht11.h> //include library of DHT11 sensor
dht11 DHT11;
#define DHT11PIN 4
float humidity;
float temperature;

String humidity_s = "";
String temperature_s = "";

#include <LBattery.h>
String battery = "";
String charging = "";

int redLightLevel = 25;
int whiteLightLevel = 18;



int trigPin1 = 6;
int echoPin1 = 7;
long timeNoSee1, wayAway1;
long tank1_depth = 25; // in cm
long tank1_level = 0;

int trigPin2 = 2;
int echoPin2 = 5;
long timeNoSee2, wayAway2;
long tank2_depth = 25; // in cm
long tank2_level = 0;

int trigPin3 = 8;
int echoPin3 = 10;
long timeNoSee3, wayAway3;
long tank3_depth = 25; // in cm
long tank3_level = 0;


uint32_t analogPin = A0;
uint32_t conductivityval = 0;




void setup()
{
  Serial.begin(115200);
  if(!LBTServer.begin((uint8_t*)"My_Hydroponics"))
  {
    Serial.println("Fail to start BT.");
    return;
  }
    Serial.println("BT server is started.");
  
  while(!LSMS.ready())
    delay(1000);
    
  pinMode(echoPin1, INPUT);
  pinMode(trigPin1, OUTPUT);
  pinMode(echoPin2, INPUT);
  pinMode(trigPin2, OUTPUT);
  pinMode(echoPin3, INPUT);
  pinMode(trigPin3, OUTPUT);
  LWiFi.begin();
  
  while(!Serial);
  // keep retrying until connected to AP
  Serial.println("Connecting to AP");
  while (0 == LWiFi.connect(WIFI_AP, LWiFiLoginInfo(WIFI_AUTH, WIFI_PASSWORD)))
  {
    delay(1000);
  }

  LTask.remoteCall(createThread, NULL);
  LTask.remoteCall(createThread1, NULL);
  LTask.remoteCall(createThread2, NULL);

}

boolean disconnectedMsg = false;

void loop()
{
  if (wayAway1 < tank1_depth) {
    tank1_level = tank1_depth - wayAway1;
  } else {
    tank1_level = 0;
  }

  if (wayAway2 < tank2_depth) {
    tank2_level = tank2_depth - wayAway2;
  } else {
    tank2_level = 0;
  }

  if (wayAway3 < tank3_depth) {
    tank3_level = tank3_depth - wayAway3;
  } else {
    tank3_level = 0;
  }
  
  char bufferH[5];
  char bufferT[5];
 
  sprintf(bufferH, "%.0f", humidity);
  sprintf(bufferT, "%.0f", temperature);

  humidity_s = String(bufferH);
  temperature_s = String(bufferT);
  

  String uploadHumidity = "humidity_display,," + humidity_s;
  String uploadTemperature = "temperature_display,," + temperature_s;
  String uploadBattery = "battery_display,," + battery;
  String uploadIsCharging = "ischarging_display,," + charging;
  String uploadLevel1 = "level1_display,," + (String) tank1_level;
  String uploadLevel2 = "level2_display,," + (String) tank2_level;
  String uploadLevel3 = "level3_display,," + (String) tank3_level;
  String uploadConductivity = "conductivity_display,," + (String) conductivityval;
  String uploadRedLevel = "redlevel_display,," + (String) redLightLevel;
  String uploadWhiteLevel = "whitelevel_display,," + (String) whiteLightLevel;

  
  String uploadData = uploadHumidity + "\n" +
                      uploadTemperature + "\n" + 
                      uploadBattery + "\n" + 
                      uploadIsCharging + "\n" +
                      uploadLevel1 + "\n" +
                      uploadLevel2 + "\n" +
                      uploadLevel3 + "\n" +
                      uploadConductivity + "\n" +
                      uploadRedLevel + "\n" +                      
                      uploadWhiteLevel + "\n";

  Serial.println(uploadData);
  
  
  Serial.println("Connecting to WebSite");


  delay(5000);
  
  while (0 == connection.connect(SITE_URL, 80))
  {
    Serial.println("Re-Connecting to WebSite");
    delay(1000);
  }

  // send HTTP request, ends with 2 CR/LF
  Serial.println("send HTTP GET request");

  connection.print("POST /mcs/v2/devices/");
  connection.print(DEVICEID);
  connection.println("/datapoints.csv HTTP/1.1");
  connection.print("Host: ");
  connection.println(SITE_URL);
  connection.print("deviceKey: ");
  connection.println(DEVICEKEY);
  connection.print("Content-Length: ");
  connection.println(uploadData.length());
  connection.println("Content-Type: text/csv");
  connection.println("Connection: close");
  connection.println();
  connection.println(uploadData);
  



  // waiting for server response
  Serial.println("waiting HTTP response:");
  while (!connection.available())
  {
    delay(100);
  }


  

  
  // Make sure we are connected, and dump the response content to Serial
  while (connection)
  {
    int v = connection.read();
    if (v != -1)
    {
      //Serial.print((char)v);
    }
    else
    {
      Serial.println("no more content, disconnect");
      connection.stop();
    }
  }

  if (!disconnectedMsg)
  {
    Serial.println("disconnected by server");
    disconnectedMsg = true;
  }
  delay(500);

  
}



boolean createThread(void* userdata) {
        // The priority can be 1 - 255 and default priority are 0
        // the arduino priority are 245
        vm_thread_create(test_thread, NULL, 0);
  return true;
}

boolean createThread1(void* userdata) {
        // The priority can be 1 - 255 and default priority are 0
        // the arduino priority are 245
        vm_thread_create(test_thread1, NULL, 0);
  return true;
}

boolean createThread2(void* userdata) {
        // The priority can be 1 - 255 and default priority are 0
        // the arduino priority are 245
        vm_thread_create(test_thread2, NULL, 0);
  return true;
}

VMINT32 test_thread(VM_THREAD_HANDLE thread_handle, void* user_data)
{
  for (;;)
  {

    
    String uploadData = "Temperature: " + temperature_s + "°C\n" +
                        "Humidity: " + humidity_s + "%\n" +
                        "Battery: " + battery + "%\n" +
                        "Battery charging: " + charging + "\n" +
                        "Tank 1 level: " + tank1_level + "\n" +
                        "Tank 2 level: " + tank2_level + "\n" +     
                        "Tank 3 level: " + tank3_level + "\n" + 
                        "Conductivity: " + conductivityval + "uS\n" +
                        "Red light level: " + whiteLightLevel + "%\n" +
                        "White light level: " + redLightLevel + "%\n";

 
      String message = "";
      message = SMS_rx();
      Serial.println(message);
      //message = message.toLowerCase();

      if (message == "info")
      {
        SMS_tx(uploadData);
      }
      else if (message == "test")
      {
        SMS_tx("Hydroponic system: ready!");
      }
      else
      {
        // do Thing C
      }


    delay(10000);
    
  }
  return 0;
}




VMINT32 test_thread1(VM_THREAD_HANDLE thread_handle, void* user_data)
{
  for (;;)
  {
    conductivityval = analogRead(analogPin);

    conductivityval = 1000 / (33 *((1024 - conductivityval)/conductivityval));  // in uS
    
    

    int chk = DHT11.read(DHT11PIN); //check DHT11 and read values
    humidity = DHT11.humidity;
    temperature = DHT11.temperature;

    battery = (String) LBattery.level();
    charging = (String) LBattery.isCharging();

    // Tank 1
    digitalWrite(trigPin1, LOW);
    delayMicroseconds(2);
    digitalWrite(trigPin1, HIGH);
    delayMicroseconds(1000);
    digitalWrite(trigPin1, LOW);
    timeNoSee1 = pulseIn(echoPin1, HIGH);
    wayAway1 = timeNoSee1 / 58.2;
    //Serial.print("distanza: ");
    //Serial.println(wayAway1);

    // Tank 2
    digitalWrite(trigPin2, LOW);
    delayMicroseconds(2);
    digitalWrite(trigPin2, HIGH);
    delayMicroseconds(1000);
    digitalWrite(trigPin2, LOW);
    timeNoSee2 = pulseIn(echoPin2, HIGH);
    wayAway2 = timeNoSee2 / 58.2;

    // Tank 3
    digitalWrite(trigPin3, LOW);
    delayMicroseconds(2);
    digitalWrite(trigPin3, HIGH);
    delayMicroseconds(1000);
    digitalWrite(trigPin3, LOW);
    timeNoSee3 = pulseIn(echoPin3, HIGH);
    wayAway3 = timeNoSee3 / 58.2;

    delay(4000);
  }
  return 0;
}


boolean SMS_tx(String msg){
    LSMS.beginSMS("0123456789");  // put the SIM number here
    LSMS.print(msg);
    if(LSMS.endSMS())
    {
      return true;
    }
    else
    {
      return false;
    }
}


VMINT32 test_thread2(VM_THREAD_HANDLE thread_handle, void* user_data)
{
  for (;;)
  {

    uint8_t buf[64];
    int bytesRead;
    String msg = "";
    
    if(LBTServer.connected())
    {
      while(true)
      {
        bytesRead = LBTServer.readBytes(buf, 64);
        if(!bytesRead)
          break;

        msg = msg + (char*)(buf);

      }
      for( int i = 0; i < sizeof(buf);  ++i )
        buf[i] = (char)0;
      Serial.print(msg);

      delay(100);
    }
    else
    {
      LBTServer.accept(5);
    }

    
    String uploadData = "Temperature: " + temperature_s + "°C\n" +
                        "Humidity: " + humidity_s + "%\n" +
                        "Battery: " + battery + "%\n" +
                        "Battery charging: " + charging + "\n" +
                        "Tank 1 level: " + tank1_level + "\n" +
                        "Tank 2 level: " + tank2_level + "\n" +     
                        "Tank 3 level: " + tank3_level + "\n" + 
                        "Conductivity: " + conductivityval + "uS\n" +
                        "Red light level: " + whiteLightLevel + "%\n" +
                        "White light level: " + redLightLevel + "%\n";


      //message = message.toLowerCase();

      if (msg == "info")
      {
        LBTServer.print(uploadData);
      }
      else if (msg == "test")
      {
        LBTServer.print("Hydroponic system: ready!");
      }
      else
      {
        // do Thing C
      }


    delay(1000);
    
  }
  return 0;
}


String SMS_rx() {
  char buf[20];
  int v;
  String msg = "";
  
  if(LSMS.available()) // Check if there is new SMS
  {

    while(true)
    {
      v = LSMS.read();
      if(v < 0)
      break;
      msg = msg + (char)v;
    }
    Serial.println();
    LSMS.flush(); // delete message
  }
  delay(1000);

  return msg;
}


